@@ -26,31 +26,31 @@
 /* list */
 struct list16
 {
-  tui16* items;
-  int count;
-  int max_count;
-  tui16 mitems[4];
+    tui16 *items;
+    int count;
+    int max_count;
+    tui16 mitems[4];
 };
 
-struct list16* APP_CC
+struct list16 *
 list16_create(void);
-void APP_CC
-list16_delete(struct list16* self);
-void APP_CC
-list16_init(struct list16* self);
-void APP_CC
-list16_deinit(struct list16* self);
-void APP_CC
-list16_add_item(struct list16* self, tui16 item);
-tui16 APP_CC
-list16_get_item(struct list16* self, int index);
-void APP_CC
-list16_clear(struct list16* self);
-int APP_CC
-list16_index_of(struct list16* self, tui16 item);
-void APP_CC
-list16_remove_item(struct list16* self, int index);
-void APP_CC
-list16_insert_item(struct list16* self, int index, tui16 item);
+void
+list16_delete(struct list16 *self);
+void
+list16_init(struct list16 *self);
+void
+list16_deinit(struct list16 *self);
+void
+list16_add_item(struct list16 *self, tui16 item);
+tui16
+list16_get_item(struct list16 *self, int index);
+void
+list16_clear(struct list16 *self);
+int
+list16_index_of(struct list16 *self, tui16 item);
+void
+list16_remove_item(struct list16 *self, int index);
+void
+list16_insert_item(struct list16 *self, int index, tui16 item);
 
 #endif
