@@ -179,6 +179,7 @@ x-special/gnome-copied-files
 #include "xcommon.h"
 #include "chansrv_fuse.h"
 #include "ms-rdpeclip.h"
+#include "xrdp_constants.h"
 
 static char g_bmp_image_header[] =
 {
@@ -245,10 +246,22 @@ static int g_cliprdr_flags = CB_USE_LONG_FORMAT_NAMES |
 static int g_formatIds[16];
 static int g_num_formatIds = 0;
 
-static int g_file_format_id = -1;
+/* Format ID assigned to "FileGroupDescriptorW" by the client */
+static int g_file_group_descriptor_format_id = -1;
 
 static char g_last_atom_name[256] = "";
 
+/*
+ * Values for the named formats we send to the client in
+ * a Format List PDU
+ */
+
+enum
+{
+    CB_FORMAT_FILE_GROUP_DESCRIPTOR = 0xc0bc
+};
+
+
 /*****************************************************************************/
 static char *
 get_atom_text(Atom atom)
@@ -646,7 +659,7 @@ clipboard_send_format_announce(int xrdp_clip_type)
             case XRDP_CB_FILE:
                 LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_send_format_announce: XRDP_CB_FILE");
                 /* canned response for "file" */
-                out_uint32_le(s, 0x0000c0bc);
+                out_uint32_le(s, CB_FORMAT_FILE_GROUP_DESCRIPTOR);
                 clipboard_out_unicode(s, "FileGroupDescriptorW", 21);
                 out_uint32_le(s, 0x0000c0ba);
                 clipboard_out_unicode(s, "FileContents", 13);
@@ -690,7 +703,7 @@ clipboard_send_format_announce(int xrdp_clip_type)
             case XRDP_CB_FILE:
                 LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_send_format_announce: XRDP_CB_FILE");
                 /* canned response for "file" */
-                out_uint32_le(s, 0x0000c0bc);
+                out_uint32_le(s, CB_FORMAT_FILE_GROUP_DESCRIPTOR);
                 out_uint8p(s, windows_native_format, sizeof(windows_native_format));
                 out_uint32_le(s, 0x0000c0ba);
                 out_uint8p(s, windows_native_format, sizeof(windows_native_format));
@@ -1028,11 +1041,11 @@ clipboard_process_format_announce(struct stream *s, int clip_msg_status,
             LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_format_announce: max formats");
         }
 
-        /* format id for file copy copy seems to keep changing */
-        /* seen 0x0000c0c8, 0x0000c0ed */
+        /* format id for file copy copy is dynamic and announced by the
+         * client */
         if (g_strcmp(desc, "FileGroupDescriptorW") == 0)
         {
-            g_file_format_id = formatId;
+            g_file_group_descriptor_format_id = formatId;
         }
     }
 
@@ -1100,48 +1113,48 @@ clipboard_process_data_request(struct stream *s, int clip_msg_status,
     in_uint32_le(s, requestedFormatId);
     switch (requestedFormatId)
     {
-        case CB_FORMAT_FILE: /* 0xC0BC */
+        case CB_FORMAT_FILE_GROUP_DESCRIPTOR:
             if ((g_clip_s2c.xrdp_clip_type == XRDP_CB_FILE) && g_clip_s2c.converted)
             {
-                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_FILE");
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_FILE_GROUP_DESCRIPTOR");
                 clipboard_send_data_response(XRDP_CB_FILE, g_clip_s2c.data,
                                              g_clip_s2c.total_bytes);
             }
             else
             {
-                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_FILE, "
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_FILE_GROUP_DESCRIPTOR, "
                           "calling XConvertSelection to g_utf8_atom");
                 g_clip_s2c.xrdp_clip_type = XRDP_CB_FILE;
                 XConvertSelection(g_display, g_clipboard_atom, g_clip_s2c.type,
                                   g_clip_property_atom, g_wnd, CurrentTime);
             }
             break;
-        case CB_FORMAT_DIB: /* 0x0008 */
+        case CF_DIB:
             if ((g_clip_s2c.xrdp_clip_type == XRDP_CB_BITMAP) && g_clip_s2c.converted)
             {
-                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_DIB");
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CF_DIB");
                 clipboard_send_data_response(XRDP_CB_BITMAP, g_clip_s2c.data,
                                              g_clip_s2c.total_bytes);
             }
             else
             {
-                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_DIB, "
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CF_DIB, "
                           "calling XConvertSelection to g_image_bmp_atom");
                 g_clip_s2c.xrdp_clip_type = XRDP_CB_BITMAP;
                 XConvertSelection(g_display, g_clipboard_atom, g_image_bmp_atom,
                                   g_clip_property_atom, g_wnd, CurrentTime);
             }
             break;
-        case CB_FORMAT_UNICODETEXT: /* 0x000D */
+        case CF_UNICODETEXT:
             if ((g_clip_s2c.xrdp_clip_type == XRDP_CB_TEXT) && g_clip_s2c.converted)
             {
-                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_UNICODETEXT");
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CF_UNICODETEXT");
                 clipboard_send_data_response(XRDP_CB_TEXT, g_clip_s2c.data,
                                              g_clip_s2c.total_bytes);
             }
             else
             {
-                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CB_FORMAT_UNICODETEXT, "
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_request: CF_UNICODETEXT, "
                           "calling XConvertSelection to g_utf8_atom");
                 g_clip_s2c.xrdp_clip_type = XRDP_CB_TEXT;
                 XConvertSelection(g_display, g_clipboard_atom, g_utf8_atom,
@@ -1157,12 +1170,15 @@ clipboard_process_data_request(struct stream *s, int clip_msg_status,
     return 0;
 }
 
-/*****************************************************************************/
-/* client to server */
-/* sent as a reply to CB_FORMAT_DATA_REQUEST; used to indicate whether
-   processing of the CB_FORMAT_DATA_REQUEST was successful; if processing
-   was successful, CB_FORMAT_DATA_RESPONSE includes contents of requested
-   clipboard data. */
+/**************************************************************************//**
+ * Process a CB_FORMAT_DATA_RESPONSE for an X client requesting an image
+ *
+ * @param s Stream containing CLIPRDR_FILELIST ([MS-RDPECLIP])
+ * @param clip_msg_status msgFlags from Clipboard PDU Header
+ * @param clip_msg_len dataLen from Clipboard PDU Header
+ *
+ * @return Status
+ */
 static int
 clipboard_process_data_response_for_image(struct stream *s,
         int clip_msg_status,
@@ -1289,6 +1305,90 @@ clipboard_process_data_response_for_file(struct stream *s,
     return rv;
 }
 
+/**************************************************************************//**
+ * Process a CB_FORMAT_DATA_RESPONSE for an X client requesting text
+ *
+ * @param s Stream containing CLIPRDR_FILELIST ([MS-RDPECLIP])
+ * @param clip_msg_status msgFlags from Clipboard PDU Header
+ * @param clip_msg_len dataLen from Clipboard PDU Header
+ *
+ * @return Status
+ */
+static int
+clipboard_process_data_response_for_text(struct stream *s,
+        int clip_msg_status,
+        int clip_msg_len)
+{
+    XSelectionRequestEvent *lxev = &g_saved_selection_req_event;
+    twchar *wtext;
+    twchar wchr;
+    int len;
+    int index;
+    int byte_count;
+
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_response_for_text: ");
+    len = (int)(s->end - s->p);
+    if (len < 1)
+    {
+        len = 0;
+    }
+    byte_count = ((len / 2) + 1) * sizeof(twchar);
+    wtext = (twchar *) g_malloc(byte_count, 0);
+    if (wtext == 0)
+    {
+        LOG(LOG_LEVEL_ERROR, "Can't allocate %d bytes for text clip response",
+            byte_count);
+
+        clipboard_refuse_selection(lxev);
+    }
+    else
+    {
+        index = 0;
+        while (s_check_rem(s, 2))
+        {
+            in_uint16_le(s, wchr);
+            wtext[index] = wchr;
+            if (wchr == 0)
+            {
+                break;
+            }
+            index++;
+        }
+        wtext[index] = 0;
+        g_free(g_clip_c2s.data);
+        g_clip_c2s.data = 0;
+        g_clip_c2s.total_bytes = 0;
+        len = g_wcstombs(0, wtext, 0);
+        if (len < 0)
+        {
+            LOG(LOG_LEVEL_ERROR,
+                "Received malformed Unicode paste text from client");
+            clipboard_refuse_selection(lxev);
+        }
+        else
+        {
+            byte_count = len + 16;
+            g_clip_c2s.data = (char *) g_malloc(byte_count, 0);
+            if (g_clip_c2s.data == 0)
+            {
+                LOG(LOG_LEVEL_ERROR,
+                    "Can't allocate %d bytes for text clip response",
+                    byte_count);
+                clipboard_refuse_selection(lxev);
+            }
+            else
+            {
+                g_wcstombs(g_clip_c2s.data, wtext, len + 1);
+                g_clip_c2s.total_bytes = g_strlen(g_clip_c2s.data);
+                g_clip_c2s.read_bytes_done = g_clip_c2s.total_bytes;
+                clipboard_provide_selection_c2s(lxev, lxev->target);
+            }
+        }
+        g_free(wtext);
+    }
+    return 0;
+}
+
 /*****************************************************************************/
 /* client to server */
 /* sent as a reply to CB_FORMAT_DATA_REQUEST; used to indicate whether
@@ -1300,75 +1400,46 @@ static int
 clipboard_process_data_response(struct stream *s, int clip_msg_status,
                                 int clip_msg_len)
 {
-    XSelectionRequestEvent *lxev;
-    twchar *wtext;
-    twchar wchr;
-    int len;
-    int index;
+    int rv = 0;
+
+    XSelectionRequestEvent *lxev = &g_saved_selection_req_event;
 
     LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_response:");
-    lxev = &g_saved_selection_req_event;
     g_clip_c2s.in_request = 0;
-    if (g_clip_c2s.xrdp_clip_type == XRDP_CB_BITMAP)
+
+    if ((clip_msg_status & CB_RESPONSE_FAIL) != 0)
+    {
+        /* Requested data was not returned from the client. Most likely
+         * the client has lost the selection between announcing it and
+         * responding to our request */
+        clipboard_refuse_selection(lxev);
+    }
+    else if ((clip_msg_status & CB_RESPONSE_OK) == 0)
+    {
+        /* One of CB_RESPONSE_FAIL or CB_RESPONSE_OK MUST be set in
+         * a CLIPRDR_FORMAT_DATA_RESPONSE msg */
+        LOG(LOG_LEVEL_ERROR, "CLIPRDR_FORMAT_DATA_RESPONSE is badly formed");
+        clipboard_refuse_selection(lxev);
+    }
+    else if (g_clip_c2s.xrdp_clip_type == XRDP_CB_BITMAP)
     {
         clipboard_process_data_response_for_image(s, clip_msg_status,
                 clip_msg_len);
-        return 0;
     }
-    if (g_clip_c2s.xrdp_clip_type == XRDP_CB_FILE)
+    else if (g_clip_c2s.xrdp_clip_type == XRDP_CB_FILE)
     {
         clipboard_process_data_response_for_file(s, clip_msg_status,
                 clip_msg_len);
-        return 0;
     }
-    LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_process_data_response: "
-              "CLIPRDR_DATA_RESPONSE");
-    len = (int)(s->end - s->p);
-    if (len < 1)
+    else
     {
-        return 0;
+        clipboard_process_data_response_for_text(s, clip_msg_status,
+                clip_msg_len);
     }
-    wtext = (twchar *) g_malloc(((len / 2) + 1) * sizeof(twchar), 0);
-    if (wtext == 0)
-    {
-        return 0;
-    }
-    index = 0;
-    while (s_check_rem(s, 2))
-    {
-        in_uint16_le(s, wchr);
-        wtext[index] = wchr;
-        if (wchr == 0)
-        {
-            break;
-        }
-        index++;
-    }
-    wtext[index] = 0;
-    g_free(g_clip_c2s.data);
-    g_clip_c2s.data = 0;
-    g_clip_c2s.total_bytes = 0;
-    len = g_wcstombs(0, wtext, 0);
-    if (len >= 0)
-    {
-        g_clip_c2s.data = (char *) g_malloc(len + 16, 0);
-        if (g_clip_c2s.data == 0)
-        {
-            g_free(wtext);
-            return 0;
-        }
-        g_wcstombs(g_clip_c2s.data, wtext, len + 1);
-    }
-    if (g_clip_c2s.data != 0)
-    {
-        g_clip_c2s.total_bytes = g_strlen(g_clip_c2s.data);
-        g_clip_c2s.read_bytes_done = g_clip_c2s.total_bytes;
-        clipboard_provide_selection_c2s(lxev, lxev->target);
-    }
-    g_free(wtext);
-    return 0;
+    return rv;
 }
 
+
 /*****************************************************************************/
 static int
 clipboard_process_clip_caps(struct stream *s, int clip_msg_status,
@@ -1417,161 +1488,6 @@ clipboard_process_clip_caps(struct stream *s, int clip_msg_status,
     return 0;
 }
 
-/*****************************************************************************/
-static int
-ss_part(char *data, int data_bytes)
-{
-    int index;
-    char *text;
-
-    LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_part: data_bytes %d read_bytes_done %d "
-              "incr_bytes_done %d", data_bytes,
-              g_clip_c2s.read_bytes_done,
-              g_clip_c2s.incr_bytes_done);
-    /* copy to buffer */
-    if (g_clip_c2s.type == g_utf8_atom)
-    {
-        /* todo unicode */
-        text = (char *)g_malloc(data_bytes, 0);
-        index = 0;
-        data_bytes /= 2;
-        while (index < data_bytes)
-        {
-            text[index] = data[index * 2];
-            index++;
-        }
-        text[index] = 0;
-        g_memcpy(g_clip_c2s.data + g_clip_c2s.read_bytes_done, text, data_bytes);
-        g_clip_c2s.read_bytes_done += data_bytes;
-        g_free(text);
-    }
-    else
-    {
-        g_memcpy(g_clip_c2s.data + g_clip_c2s.read_bytes_done, data, data_bytes);
-        g_clip_c2s.read_bytes_done += data_bytes;
-    }
-    if (g_clip_c2s.incr_in_progress)
-    {
-        LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_part: incr_in_progress set");
-        return 0;
-    }
-    if (g_clip_c2s.read_bytes_done <= g_clip_c2s.incr_bytes_done)
-    {
-        LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_part: read_bytes_done < incr_bytes_done");
-        return 0;
-    }
-    data = g_clip_c2s.data + g_clip_c2s.incr_bytes_done;
-    data_bytes = g_clip_c2s.read_bytes_done - g_clip_c2s.incr_bytes_done;
-    if (data_bytes > g_incr_max_req_size)
-    {
-        data_bytes = g_incr_max_req_size;
-    }
-    g_clip_c2s.incr_bytes_done += data_bytes;
-    XChangeProperty(g_display, g_clip_c2s.window,
-                    g_clip_c2s.property, g_clip_c2s.type, 8,
-                    PropModeReplace, (tui8 *)data, data_bytes);
-    g_clip_c2s.incr_in_progress = 1;
-    return 0;
-}
-
-/*****************************************************************************/
-static int
-ss_end(void)
-{
-    char *data;
-    int data_bytes;
-
-    LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_end:");
-    g_clip_c2s.doing_response_ss = 0;
-    g_clip_c2s.in_request = 0;
-
-    if (g_clip_c2s.incr_in_progress)
-    {
-        LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_end: incr_in_progress set");
-        return 0;
-    }
-    if (g_clip_c2s.read_bytes_done <= g_clip_c2s.incr_bytes_done)
-    {
-        LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_end: read_bytes_done < incr_bytes_done");
-        return 0;
-    }
-    data = g_clip_c2s.data + g_clip_c2s.incr_bytes_done;
-    data_bytes = g_clip_c2s.read_bytes_done - g_clip_c2s.incr_bytes_done;
-    if (data_bytes > g_incr_max_req_size)
-    {
-        data_bytes = g_incr_max_req_size;
-    }
-    g_clip_c2s.incr_bytes_done += data_bytes;
-    XChangeProperty(g_display, g_clip_c2s.window,
-                    g_clip_c2s.property, g_clip_c2s.type, 8,
-                    PropModeReplace, (tui8 *)data, data_bytes);
-    g_clip_c2s.incr_in_progress = 1;
-    return 0;
-}
-
-/*****************************************************************************/
-static int
-ss_start(char *data, int data_bytes, int total_bytes)
-{
-    XEvent xev;
-    XSelectionRequestEvent *req;
-    long val1[2];
-    int incr_bytes;
-
-    LOG_DEVEL(LOG_LEVEL_DEBUG, "ss_start: data_bytes %d total_bytes %d",
-              data_bytes, total_bytes);
-    req = &g_saved_selection_req_event;
-
-    incr_bytes = total_bytes;
-    if (req->target == g_image_bmp_atom)
-    {
-        incr_bytes += 14;
-    }
-    else if (req->target == g_utf8_atom)
-    {
-        incr_bytes /= 2;
-    }
-    val1[0] = incr_bytes; /* a guess */
-    val1[1] = 0;
-
-    g_clip_c2s.doing_response_ss = 1;
-    g_clip_c2s.incr_bytes_done = 0;
-    g_clip_c2s.read_bytes_done = 0;
-    g_clip_c2s.type = req->target;
-    g_clip_c2s.property = req->property;
-    g_clip_c2s.window = req->requestor;
-    g_free(g_clip_c2s.data);
-    g_clip_c2s.data = (char *)g_malloc(incr_bytes + 64, 0);
-    g_clip_c2s.total_bytes = incr_bytes;
-
-    XChangeProperty(g_display, req->requestor, req->property,
-                    g_incr_atom, 32, PropModeReplace, (tui8 *)val1, 1);
-    /* we need events from that other window */
-    XSelectInput(g_display, req->requestor, PropertyChangeMask);
-    g_memset(&xev, 0, sizeof(xev));
-    xev.xselection.type = SelectionNotify;
-    xev.xselection.send_event = True;
-    xev.xselection.display = req->display;
-    xev.xselection.requestor = req->requestor;
-    xev.xselection.selection = req->selection;
-    xev.xselection.target = req->target;
-    xev.xselection.property = req->property;
-    xev.xselection.time = req->time;
-    XSendEvent(g_display, req->requestor, False, NoEventMask, &xev);
-
-    if (req->target == g_image_bmp_atom)
-    {
-        g_memcpy(g_clip_c2s.data, g_bmp_image_header, 14);
-        g_clip_c2s.read_bytes_done = 14;
-    }
-
-    g_clip_c2s.incr_in_progress = 1;
-
-    ss_part(data, data_bytes);
-
-    return 0;
-}
-
 /*****************************************************************************/
 int
 clipboard_data_in(struct stream *s, int chan_id, int chan_flags, int length,
@@ -1582,7 +1498,6 @@ clipboard_data_in(struct stream *s, int chan_id, int chan_flags, int length,
     int clip_msg_status;
     int rv;
     struct stream *ls;
-    char *holdp;
 
     if (!g_clip_up)
     {
@@ -1598,37 +1513,6 @@ clipboard_data_in(struct stream *s, int chan_id, int chan_flags, int length,
               chan_id, chan_flags, length, total_length,
               g_clip_c2s.in_request, g_ins->size);
 
-    if (g_clip_c2s.doing_response_ss)
-    {
-        ss_part(s->p, length);
-        if ((chan_flags & 3) == 2)
-        {
-            LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_data_in: calling ss_end");
-            ss_end();
-        }
-        return 0;
-    }
-
-    if (g_clip_c2s.in_request)
-    {
-        if (total_length > 32 * 1024)
-        {
-            if ((chan_flags & 3) == 1)
-            {
-                holdp = s->p;
-                in_uint16_le(s, clip_msg_id);
-                in_uint16_le(s, clip_msg_status);
-                in_uint32_le(s, clip_msg_len);
-                if (clip_msg_id == CB_FORMAT_DATA_RESPONSE)
-                {
-                    ss_start(s->p, length - 8, total_length - 8);
-                    return 0;
-                }
-                s->p = holdp;
-            }
-        }
-    }
-
     if ((chan_flags & 3) == 3)
     {
         ls = s;
@@ -2295,21 +2179,30 @@ clipboard_event_selection_request(XEvent *xevent)
         atom_buf[1] = g_timestamp_atom;
         atom_buf[2] = g_multiple_atom;
         atom_count = 3;
-        if ((g_cfg->restrict_inbound_clipboard & CLIP_RESTRICT_TEXT) == 0)
+
+        /* Only announce text if the client is advertising text, or
+         * a file list */
+        if (clipboard_find_format_id(CF_UNICODETEXT) >= 0 ||
+                clipboard_find_format_id(CF_OEMTEXT) >= 0 ||
+                clipboard_find_format_id(CF_TEXT) >= 0 ||
+                clipboard_find_format_id(g_file_group_descriptor_format_id) >= 0)
         {
-            atom_buf[atom_count] = XA_STRING;
-            atom_count++;
-            atom_buf[atom_count] = g_utf8_atom;
-            atom_count++;
+            if ((g_cfg->restrict_inbound_clipboard & CLIP_RESTRICT_TEXT) == 0)
+            {
+                atom_buf[atom_count] = XA_STRING;
+                atom_count++;
+                atom_buf[atom_count] = g_utf8_atom;
+                atom_count++;
+            }
         }
-        if (clipboard_find_format_id(CB_FORMAT_DIB) >= 0 &&
+        if (clipboard_find_format_id(CF_DIB) >= 0 &&
                 (g_cfg->restrict_inbound_clipboard & CLIP_RESTRICT_IMAGE) == 0)
         {
             LOG_DEVEL(LOG_LEVEL_DEBUG, "  reporting image/bmp");
             atom_buf[atom_count] = g_image_bmp_atom;
             atom_count++;
         }
-        if (clipboard_find_format_id(g_file_format_id) >= 0 &&
+        if (clipboard_find_format_id(g_file_group_descriptor_format_id) >= 0 &&
                 (g_cfg->restrict_inbound_clipboard & CLIP_RESTRICT_FILE) == 0)
         {
             LOG_DEVEL(LOG_LEVEL_DEBUG, "  reporting text/uri-list");
@@ -2353,7 +2246,7 @@ clipboard_event_selection_request(XEvent *xevent)
     }
     else if ((lxev->target == XA_STRING) || (lxev->target == g_utf8_atom))
     {
-        if (clipboard_find_format_id(g_file_format_id) >= 0)
+        if (clipboard_find_format_id(g_file_group_descriptor_format_id) >= 0)
         {
             LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_event_selection_request: "
                       "text requested when files available");
@@ -2371,7 +2264,7 @@ clipboard_event_selection_request(XEvent *xevent)
                          sizeof(g_saved_selection_req_event));
                 g_clip_c2s.type = lxev->target;
                 g_clip_c2s.xrdp_clip_type = XRDP_CB_FILE;
-                clipboard_send_data_request(g_file_format_id);
+                clipboard_send_data_request(g_file_group_descriptor_format_id);
             }
 
         }
@@ -2387,11 +2280,14 @@ clipboard_event_selection_request(XEvent *xevent)
             }
             else
             {
+                /* The client may have advertised CF_TEXT or CF_OEMTEXT,
+                 * but the Windows clipboard will convert these formats
+                 * to Unicode if asked */
                 g_memcpy(&g_saved_selection_req_event, lxev,
                          sizeof(g_saved_selection_req_event));
                 g_clip_c2s.type = lxev->target;
                 g_clip_c2s.xrdp_clip_type = XRDP_CB_TEXT;
-                clipboard_send_data_request(CB_FORMAT_UNICODETEXT);
+                clipboard_send_data_request(CF_UNICODETEXT);
             }
 
         }
@@ -2430,7 +2326,7 @@ clipboard_event_selection_request(XEvent *xevent)
                      sizeof(g_saved_selection_req_event));
             g_clip_c2s.type = g_image_bmp_atom;
             g_clip_c2s.xrdp_clip_type = XRDP_CB_BITMAP;
-            clipboard_send_data_request(CB_FORMAT_DIB);
+            clipboard_send_data_request(CF_DIB);
         }
         return 0;
 
@@ -2467,7 +2363,7 @@ clipboard_event_selection_request(XEvent *xevent)
                      sizeof(g_saved_selection_req_event));
             g_clip_c2s.type = g_file_atom1;
             g_clip_c2s.xrdp_clip_type = XRDP_CB_FILE;
-            clipboard_send_data_request(g_file_format_id);
+            clipboard_send_data_request(g_file_group_descriptor_format_id);
             return 0;
         }
 
@@ -2505,7 +2401,7 @@ clipboard_event_selection_request(XEvent *xevent)
                      sizeof(g_saved_selection_req_event));
             g_clip_c2s.type = g_file_atom2;
             g_clip_c2s.xrdp_clip_type = XRDP_CB_FILE;
-            clipboard_send_data_request(g_file_format_id);
+            clipboard_send_data_request(g_file_group_descriptor_format_id);
             return 0;
         }
     }
@@ -2676,14 +2572,12 @@ clipboard_event_property_notify(XEvent *xevent)
             format_in_bytes = FORMAT_TO_BYTES(actual_format_return);
             new_data_len = nitems_returned * format_in_bytes;
             cptr = (char *) g_malloc(g_clip_s2c.total_bytes + new_data_len, 0);
-            g_memcpy(cptr, g_clip_s2c.data, g_clip_s2c.total_bytes);
-            g_free(g_clip_s2c.data);
-
             if (cptr == NULL)
             {
+                /* cannot add any more data */
+                g_free(g_clip_s2c.data);
                 g_clip_s2c.data = 0;
 
-                /* cannot add any more data */
                 if (data != 0)
                 {
                     XFree(data);
@@ -2692,14 +2586,15 @@ clipboard_event_property_notify(XEvent *xevent)
                 XDeleteProperty(g_display, g_wnd, g_clip_s2c.property);
                 return 0;
             }
+            g_memcpy(cptr, g_clip_s2c.data, g_clip_s2c.total_bytes);
+            g_free(g_clip_s2c.data);
 
             LOG_DEVEL(LOG_LEVEL_DEBUG, "clipboard_event_property_notify: new_data_len %d", new_data_len);
             g_clip_s2c.data = cptr;
-            g_memcpy(g_clip_s2c.data + g_clip_s2c.total_bytes, data, new_data_len);
-            g_clip_s2c.total_bytes += new_data_len;
-
             if (data)
             {
+                g_memcpy(g_clip_s2c.data + g_clip_s2c.total_bytes, data, new_data_len);
+                g_clip_s2c.total_bytes += new_data_len;
                 XFree(data);
             }
 
